'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Load security, middleware, service and api implementations
 */
const fs = require('fs');
const path = require('path');

const debug = require('debug')('ruo');
const moder = require('moder');
const _ = require('lodash');
const Waterline = require('waterline');
const pascalcase = require('uppercamelcase');
const promiseify = require('denodeify');

const rc = require('./rc');

var _require = require('./utility');

const isTest = _require.isTest,
      wrapMiddleware = _require.wrapMiddleware;


const waterline = new Waterline();
const initialize = promiseify(waterline.initialize.bind(waterline));

exports.initialize = (() => {
  var _ref = _asyncToGenerator(function* () {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    let modelConfig = _ref2.model;

    const globals = {};
    // load model, service and security
    ['model', 'service', 'security', 'middleware'].forEach(function (type) {
      const name = type + 's';
      globals[name] = {};
      try {
        const dir = path.join(rc.target, type);
        fs.statSync(dir);
        const modules = moder(dir, {
          lazy: false,
          filter: isTest
        });
        debug(type, Object.keys(modules));
        globals[name] = modules;
      } catch (err) {
        if (err.code !== 'ENOENT') {
          throw err;
        }
      }
    });

    // wrap security middlewares
    _.forEach(globals.securitys, function (middleware, name) {
      // TODO: what if security middleware require arguments?
      globals.securitys[name] = wrapMiddleware(middleware());
    });

    // wrap normal middlewares
    _.forEach(globals.middlewares, function (middleware, name) {
      globals.middlewares[name] = function () {
        wrapMiddleware(middleware.apply(undefined, arguments));
      };
    });

    // initialize models
    if (modelConfig) {
      _.forEach(globals.models, function (model) {
        // FIXME: why modelrc.defaults not working?
        waterline.loadCollection(Waterline.Collection.extend(_.merge({}, modelConfig.defaults, model)));
      });
      for (let name in modelConfig.adapters) {
        modelConfig.adapters[name] = require(modelConfig.adapters[name]);
      }
      const ontology = yield initialize(modelConfig);
      const models = _.reduce(ontology.collections, function (obj, model, name) {
        name = pascalcase(name);
        obj[name] = model;
        return obj;
      }, {});
      // TODO: just pick one of the model, any better way?
      const Model = models[Object.keys(models)[0]];
      models.query = promiseify(Model.query.bind(Model));
      globals.models = models;
    }

    return globals;
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();