'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('chai');

const expect = _require.expect;

const _ = require('lodash');

const validateAsync = require('./definition-validator');
const SPEC_TEMPLATE = {
  swagger: '2.0',
  info: { version: '0.0.0', title: 'Example RESTful API' },
  host: 'localhost:8088'
};

describe('swagger/definition-validator', () => {
  it('should verify paths exist', _asyncToGenerator(function* () {
    yield expect(validateAsync(SPEC_TEMPLATE)).to.be.rejectedWith('Missing required property: paths');

    let definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {};
    yield expect(validateAsync(definition)).to.not.be.rejected;
  }));

  it('should verify responses exist', _asyncToGenerator(function* () {
    let definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {}
      }
    };
    yield expect(validateAsync(definition)).to.be.rejectedWith('Missing required property: responses');
  }));

  it('should verify responses status code exist', _asyncToGenerator(function* () {
    let definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          responses: {}
        }
      }
    };
    yield expect(validateAsync(definition)).to.be.rejectedWith('Too few properties defined');
  }));

  it('should verify security being array', _asyncToGenerator(function* () {
    let definition;

    definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          security: {},
          responses: {
            default: {
              schema: {}
            }
          }
        }
      }
    };
    yield expect(validateAsync(definition)).to.be.rejectedWith('Expected type array but found type object');

    definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          security: [],
          responses: {
            default: {
              schema: {}
            }
          }
        }
      }
    };
    yield expect(validateAsync(definition)).to.not.be.rejected;
  }));

  it('should verify responses.default.schema exist', _asyncToGenerator(function* () {
    let definition;

    definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          responses: {
            200: {}
          }
        }
      }
    };
    yield expect(validateAsync(definition)).to.be.rejectedWith('responses.default.schema required');

    definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          responses: {
            default: {
              schema: {}
            }
          }
        }
      }
    };
    yield expect(validateAsync(definition)).to.not.be.rejected;
  }));

  it('should verify parameter and response x-examples', _asyncToGenerator(function* () {
    let definition;

    definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          responses: {
            default: {
              'x-examples': {
                field: 42
              },
              schema: {
                type: 'object',
                properties: {
                  field: {
                    type: 'string'
                  }
                }
              }
            }
          }
        }
      }
    };
    yield expect(validateAsync(definition)).to.be.rejectedWith('InvalidSchemaExample');

    definition = _.clone(SPEC_TEMPLATE);
    definition.paths = {
      '/path': {
        get: {
          responses: {
            default: {
              'x-examples': {
                field: 'stringValue'
              },
              schema: {
                type: 'object',
                properties: {
                  field: {
                    type: 'string'
                  }
                }
              }
            }
          }
        }
      }
    };
    yield expect(validateAsync(definition)).to.not.be.rejected;
  }));
});