'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('chai');

const expect = _require.expect;


const Path = require('./path');
const sampleDef = require('./sample-definition.json');

describe('swagger/path', () => {
  const path = '/pet/{petId}';
  const pathDef = sampleDef.paths[path];
  const pathObject = new Path(path, pathDef, { basePathPrefix: '' }, ['paths', path]);

  it('should create express regular express to match path', () => {
    expect(pathObject.regexp.test('/pet/1')).to.eql(true);
    expect(pathObject.regexp.test('/pet/abc')).to.eql(true);
    expect(pathObject.regexp.test('/pet/1/')).to.eql(true);
    expect(pathObject.regexp.test('/pet')).to.eql(false);
    expect(pathObject.regexp.test('/pet/1/2')).to.eql(false);
  });

  it('should create operation objects', () => {
    expect(pathObject.operationObjects.length).to.eql(Object.keys(pathDef).length);
  });

  it('should get operation by method', () => {
    const method = Object.keys(pathDef)[0];
    const operationObject = pathObject.getOperation(method);
    expect(operationObject.method).to.eql(method);
  });

  it('should return get operation object if method is head', _asyncToGenerator(function* () {
    const operationObject = pathObject.getOperation('head');
    expect(operationObject.method).to.eql('get');
  }));
});